@@ -296,7 +296,9 @@ def optional_user_sources_accepted(
     return wrapper
 
 
-def optional_prior_accepted(ex_post: bool = False, infer_missing: bool = True):
+def optional_prior_accepted(
+    ex_post: bool = False, infer_missing: bool = True, infer_missing_play: bool = False
+):
     """Decorator which specifies that a GET or POST request accepts an optional prior.
     It parses relevant form data and sets the "prior" keyword param.
 
@@ -304,9 +306,15 @@ def optional_prior_accepted(ex_post: bool = False, infer_missing: bool = True):
     -   Denotes "at least before <prior>"
     -   This results in the filter belief_time_window = (None, prior)
 
-    Optionally, an ex_post flag can be passed to the decorator to indicate that only ex-post datetimes are allowed.
-    As a useful setting (at least for POST requests), set infer_missing to True to have servers
-    (that are not in play mode) derive a prior from the server time.
+    Interpretation for POST requests:
+    -   Denotes "recorded <prior> to some datetime,
+    -   this results in the assignment belief_time = prior
+
+    :param ex_post:                     if True, only ex-post datetimes are allowed.
+    :param infer_missing:               if True, servers assume that the belief_time of posted
+                                        values is server time. This setting is meant to be used for POST requests.
+    :param infer_missing_play:          if True, servers in play mode assume that the belief_time of posted
+                                        values is server time. This setting is meant to be used for POST requests.
     """
 
     def wrapper(fn):
@@ -332,11 +340,11 @@ def optional_prior_accepted(ex_post: bool = False, infer_missing: bool = True):
                     if prior < knowledge_time:
                         extra_info = "Meter data can only be observed after the fact."
                         return invalid_horizon(extra_info)
-            elif (
-                infer_missing is True
-                and current_app.config.get("FLEXMEASURES_MODE", "") != "play"
+            elif infer_missing is True or (
+                infer_missing_play is True
+                and current_app.config.get("FLEXMEASURES_MODE", "") == "play"
             ):
-                # A missing prior is inferred by the server (if not in play mode)
+                # A missing prior is inferred by the server
                 prior = server_now()
             else:
                 # Otherwise, a missing prior is fine (a horizon may still be inferred by the server)
@@ -353,6 +361,7 @@ def optional_prior_accepted(ex_post: bool = False, infer_missing: bool = True):
 def optional_horizon_accepted(  # noqa C901
     ex_post: bool = False,
     infer_missing: bool = True,
+    infer_missing_play: bool = False,
     accept_repeating_interval: bool = False,
 ):
     """Decorator which specifies that a GET or POST request accepts an optional horizon.
@@ -376,11 +385,13 @@ def optional_horizon_accepted(  # noqa C901
         def post_meter_data(horizon):
             return 'Meter data posted'
 
-    :param ex_post:                   if True, only non-positive horizons are allowed.
-    :param infer_missing:             if True, servers that are in play mode assume that the belief_horizon of posted
-                                      values is 0 hours. This setting is meant to be used for POST requests.
-    :param accept_repeating_interval: if True, the "rolling" keyword param is also set
-                                      (this was used for POST requests before v2.0)
+    :param ex_post:                     if True, only non-positive horizons are allowed.
+    :param infer_missing:               if True, servers assume that the belief_horizon of posted
+                                        values is 0 hours. This setting is meant to be used for POST requests.
+    :param infer_missing_play:          if True, servers in play mode assume that the belief_horizon of posted
+                                        values is 0 hours. This setting is meant to be used for POST requests.
+    :param accept_repeating_interval:   if True, the "rolling" keyword param is also set
+                                        (this was used for POST requests before v2.0)
     """
 
     def wrapper(fn):
@@ -410,15 +421,12 @@ def optional_horizon_accepted(  # noqa C901
                         "For example: R/P1D should be replaced by P1D."
                     )
                     return invalid_horizon(extra_info)
-            elif (
-                infer_missing is True
+            elif infer_missing is True or (
+                infer_missing_play is True
                 and current_app.config.get("FLEXMEASURES_MODE", "") == "play"
             ):
-                # A missing horizon is set to zero for servers in play mode
+                # A missing horizon is set to zero
                 horizon = timedelta(hours=0)
-            elif infer_missing is True and accept_repeating_interval is True:
-                extra_info = "Missing horizons are no longer accepted for API versions below v2.0."
-                return invalid_horizon(extra_info)
             else:
                 # Otherwise, a missing horizon is fine (a prior may still be inferred by the server)
                 horizon = None
