@@ -1,7 +1,7 @@
 """CLI Tasks for (de)populating the database - most useful in development"""
 
 from datetime import timedelta
-from typing import List
+from typing import List, Optional
 
 import pandas as pd
 import pytz
@@ -10,13 +10,21 @@ from flask.cli import with_appcontext
 from flask_security.utils import hash_password
 import click
 import getpass
+from sqlalchemy.exc import IntegrityError
+import timely_beliefs as tb
 
+from flexmeasures.data import db
 from flexmeasures.data.services.forecasting import create_forecasting_jobs
 from flexmeasures.data.services.users import create_user
-from flexmeasures.data.models.time_series import Sensor, SensorSchema
+from flexmeasures.data.models.time_series import Sensor, SensorSchema, TimedBelief
 from flexmeasures.data.models.assets import Asset, AssetSchema
 from flexmeasures.data.models.markets import Market
 from flexmeasures.data.models.weather import WeatherSensor, WeatherSensorSchema
+from flexmeasures.data.models.data_sources import (
+    get_or_create_source,
+    get_source_or_none,
+)
+from flexmeasures.utils.time_utils import server_now
 
 
 @click.group("add")
@@ -24,6 +32,11 @@ def fm_add_data():
     """FlexMeasures: Add data."""
 
 
+@click.group("dev-add")
+def fm_dev_add_data():
+    """Developer CLI commands not yet meant for users: Add data."""
+
+
 @fm_add_data.command("user")
 @with_appcontext
 @click.option("--username", required=True)
@@ -63,7 +76,7 @@ def new_user(username: str, email: str, roles: List[str], timezone: str):
     print(f"Successfully created user {created_user}")
 
 
-@fm_add_data.command("sensor")
+@fm_dev_add_data.command("sensor")
 @with_appcontext
 @click.option("--name", required=True)
 @click.option("--unit", required=True, help="e.g. °C, m/s, kW/m²")
@@ -201,6 +214,154 @@ def add_initial_structure():
     populate_structure(app.db)
 
 
+@fm_dev_add_data.command("beliefs")
+@with_appcontext
+@click.argument("file", type=click.Path(exists=True))
+@click.option(
+    "--sensor-id",
+    required=True,
+    type=click.IntRange(min=1),
+    help="Sensor to which the beliefs pertain.",
+)
+@click.option(
+    "--source",
+    required=True,
+    type=str,
+    help="Source of the beliefs (an existing source id or name, or a new name).",
+)
+@click.option(
+    "--horizon",
+    required=False,
+    type=int,
+    help="Belief horizon in minutes (use positive horizon for ex-ante beliefs or negative horizon for ex-post beliefs).",
+)
+@click.option(
+    "--cp",
+    required=False,
+    type=click.FloatRange(0, 1),
+    help="Cumulative probability in the range [0, 1].",
+)
+@click.option(
+    "--allow-overwrite/--do-not-allow-overwrite",
+    default=False,
+    help="Allow overwriting possibly already existing data.\n"
+    "Not allowing overwriting can be much more efficient",
+)
+@click.option(
+    "--skiprows",
+    required=False,
+    default=1,
+    type=int,
+    help="Number of rows to skip from the top. Set to >1 to skip additional headers.",
+)
+@click.option(
+    "--nrows",
+    required=False,
+    type=int,
+    help="Number of rows to read (from the top, after possibly skipping rows). Leave out to read all rows.",
+)
+@click.option(
+    "--datecol",
+    required=False,
+    default=0,
+    type=int,
+    help="Column number with datetimes (0 is 1st column, the default)",
+)
+@click.option(
+    "--valuecol",
+    required=False,
+    default=1,
+    type=int,
+    help="Column number with values (1 is 2nd column, the default)",
+)
+@click.option(
+    "--sheet_number",
+    required=False,
+    type=int,
+    help="[For xls or xlsx files] Sheet number with the data (0 is 1st sheet)",
+)
+def add_beliefs(
+    file: str,
+    sensor_id: int,
+    source: str,
+    horizon: Optional[int] = None,
+    cp: Optional[float] = None,
+    allow_overwrite: bool = False,
+    skiprows: int = 1,
+    nrows: Optional[int] = None,
+    datecol: int = 0,
+    valuecol: int = 1,
+    sheet_number: Optional[int] = None,
+):
+    """Add sensor data from a csv file (also accepts xls or xlsx).
+
+    To use default settings, structure your csv file as follows:
+
+        - One header line (will be ignored!)
+        - UTC datetimes in 1st column
+        - values in 2nd column
+
+    For example:
+
+        Date,Inflow (cubic meter)
+        2020-12-03 14:00,212
+        2020-12-03 14:10,215.6
+        2020-12-03 14:20,203.8
+
+    In case no --horizon is specified, the moment of executing this CLI command is taken
+    as the time at which the beliefs were recorded.
+    """
+    sensor = Sensor.query.filter(Sensor.id == sensor_id).one_or_none()
+    if sensor is None:
+        print(f"Failed to create beliefs: no sensor found with id {sensor_id}.")
+        return
+    if source.isdigit():
+        _source = get_source_or_none(int(source), source_type="CLI script")
+        if not _source:
+            print(f"Failed to find source {source}.")
+            return
+    else:
+        _source = get_or_create_source(source, source_type="CLI script")
+
+    # Set up optional parameters for read_csv
+    kwargs = dict()
+    if file.split(".")[-1].lower() == "csv":
+        kwargs["infer_datetime_format"] = True
+    if sheet_number is not None:
+        kwargs["sheet_name"] = sheet_number
+    if horizon is not None:
+        kwargs["belief_horizon"] = timedelta(minutes=horizon)
+    else:
+        kwargs["belief_time"] = server_now().astimezone(pytz.timezone(sensor.timezone))
+
+    bdf = tb.read_csv(
+        file,
+        sensor,
+        source=_source,
+        cumulative_probability=cp,
+        header=None,
+        skiprows=skiprows,
+        nrows=nrows,
+        usecols=[datecol, valuecol],
+        parse_dates=True,
+        **kwargs,
+    )
+    try:
+        TimedBelief.add(
+            bdf,
+            expunge_session=True,
+            allow_overwrite=allow_overwrite,
+            bulk_save_objects=True,
+            commit_transaction=True,
+        )
+        print(f"Successfully created beliefs\n{bdf}")
+    except IntegrityError as e:
+        db.session.rollback()
+        print(f"Failed to create beliefs due to the following error: {e.orig}")
+        if not allow_overwrite:
+            print("As a possible workaround, use the --allow-overwrite flag.")
+
+
 @fm_add_data.command("forecasts")
 @with_appcontext
 @click.option(
@@ -230,14 +391,14 @@ def add_initial_structure():
     multiple=True,
     type=click.Choice(["1", "6", "24", "48"]),
     default=["1", "6", "24", "48"],
-    help="Forecasting horizon in hours. This argument can be given multiple times.",
+    help="Forecasting horizon in hours. This argument can be given multiple times. Defaults to all possible horizons.",
 )
 @click.option(
     "--as-job",
     is_flag=True,
     help="Whether to queue a forecasting job instead of computing directly."
     " Useful to run locally and create forecasts on a remote server. In that case, just point the redis db in your"
-    " config settings to that of the remote server. To process the job, run a worker to process the forecasting queue.",
+    " config settings to that of the remote server. To process the job, run a worker to process the forecasting queue. Defaults to False.",
 )
 def create_forecasts(
     asset_type: str = None,
@@ -338,6 +499,7 @@ def collect_weather_data(region, location, num_cells, method, store_in_db):
 
 
 app.cli.add_command(fm_add_data)
+app.cli.add_command(fm_dev_add_data)
 
 
 def check_timezone(timezone):
